@@ -1,9 +1,7 @@
-#' @title Add package file
-#'
-#' @description
-#' Automatically generate a file containing functions and related code for R package development.
+#' @title Add a package file and print package information
 #'
 #' @md
+#' @inheritParams log_message
 #' @param use_figlet Whether to use figlet for ASCII art generation.
 #' Default is `TRUE`. Details see [figlet].
 #' @param figlet_font Character string, figlet font to use.
@@ -11,11 +9,9 @@
 #' @param colors Character vector, colors to use for the logo elements.
 #' @param unicode Whether to use Unicode symbols.
 #' Default is `TRUE`.
-#' @param verbose Whether to print progress messages.
-#' Default is `TRUE`.
 #'
 #' @return
-#' Creates a file in specified output directory.
+#' Creates a file named `R/<pkg_name>-package.R`.
 #'
 #' @export
 add_pkg_file <- function(
@@ -30,7 +26,7 @@ add_pkg_file <- function(
   desc_file <- "DESCRIPTION"
   pkgdown_file <- "_pkgdown.yml"
 
-  desc_info <- .read_description(desc_file, verbose)
+  desc_info <- read_description(desc_file, verbose)
 
   pkg_name <- desc_info$Package
   title <- desc_info$Title
@@ -118,8 +114,6 @@ add_pkg_file <- function(
     )
   }
 
-  has_src_dir <- dir.exists("src")
-
   file_content <- generate_content(
     pkg_name = pkg_name,
     title = title,
@@ -130,7 +124,7 @@ add_pkg_file <- function(
     ascii_lines = ascii_lines,
     colors = colors,
     unicode = unicode,
-    has_src_dir = has_src_dir
+    src_exist = dir.exists("src")
   )
 
   output_file <- file.path(
@@ -142,8 +136,8 @@ add_pkg_file <- function(
     message_type = "success",
     verbose = verbose
   )
-  .check_dependency(desc_file, verbose)
-  .check_pkgdown(pkgdown_file, pkg_name, verbose)
+  check_dependencies(desc_file, verbose)
+  check_pkgdown(pkgdown_file, pkg_name, verbose)
 
   invisible(file_content)
 }
@@ -158,13 +152,13 @@ generate_content <- function(
     ascii_lines,
     colors,
     unicode,
-    has_src_dir = FALSE) {
-  ascii_with_numbers <- .add_color_numbers(
+    src_exist = FALSE) {
+  ascii_with_numbers <- add_ascii_numbers(
     ascii_lines,
     length(colors)
   )
 
-  use_dynlib_line <- if (has_src_dir) {
+  use_dynlib_line <- if (src_exist) {
     c(
       paste0("#' @useDynLib ", pkg_name),
       "#'"
@@ -226,7 +220,7 @@ generate_content <- function(
     ),
     "  )",
     "",
-    .generate_hexa(length(colors), colors, unicode),
+    generate_hexa(length(colors), colors, unicode),
     "",
     "  col_hexa <- mapply(",
     "    function(x, y) cli::make_ansi_style(y)(x),",
@@ -261,7 +255,6 @@ generate_content <- function(
     ),
     "#'",
     "#' @export",
-    "#'",
     paste0(
       "print.", tolower(pkg_name), "_logo <- function(x, ...) {"
     ),
@@ -272,16 +265,19 @@ generate_content <- function(
     ".onAttach <- function(libname, pkgname) {",
     "  verbose <- thisutils::get_verbose()",
     "  if (isTRUE(verbose)) {",
-    "    version <- utils::packageDescription(",
-    "      pkgname,",
-    "      fields = \"Version\"",
+    "    version <- utils::packageVersion(pkgname)",
+    "    date <- utils::packageDate(pkgname)",
+    "    url <- utils::packageDescription(",
+    "      pkgname, fields = \"URL\"",
     "    )",
     "",
     "    msg <- paste0(",
     "      cli::col_grey(strrep(\"-\", 60)),",
     "      \"\\n\",",
-    "      cli::col_blue(pkgname, \" version \", version),",
+    "      cli::col_blue(\"Version: \", version, \" (\", date, \" update)\"),",
     "      \"\\n\",",
+    "      cli::col_blue(\"Website: \", cli::style_italic(url)),",
+    "      \"\\n\\n\",",
     "      cli::col_grey(\"This message can be suppressed by:\"),",
     "      \"\\n\",",
     paste0(
@@ -308,7 +304,7 @@ generate_content <- function(
   content
 }
 
-.add_color_numbers <- function(
+add_ascii_numbers <- function(
     ascii_lines,
     num_colors) {
   if (length(ascii_lines) == 0) {
@@ -324,7 +320,7 @@ generate_content <- function(
   result
 }
 
-.generate_hexa <- function(
+generate_hexa <- function(
     num_colors,
     colors,
     unicode) {
@@ -359,7 +355,7 @@ generate_content <- function(
   code
 }
 
-.read_description <- function(
+read_description <- function(
     desc_file,
     verbose = TRUE) {
   if (!file.exists(desc_file)) {
@@ -534,7 +530,7 @@ generate_content <- function(
   )
 }
 
-.check_dependency <- function(desc_file, verbose = TRUE) {
+check_dependencies <- function(desc_file, verbose = TRUE) {
   if (!file.exists(desc_file)) {
     log_message(
       "{.file {desc_file}} not found",
@@ -545,8 +541,8 @@ generate_content <- function(
 
   desc_content <- readLines(desc_file, warn = FALSE)
 
-  imports_start <- which(grepl("^Imports:", desc_content))
-  if (length(imports_start) == 0) {
+  imports_bounds <- find_section(desc_content, "Imports")
+  if (is.null(imports_bounds)) {
     log_message(
       "No {.cls Imports} section found in {.file {desc_file}}",
       message_type = "warning",
@@ -555,51 +551,96 @@ generate_content <- function(
     return()
   }
 
-  imports_end <- imports_start
-  for (i in (imports_start + 1):length(desc_content)) {
-    if (grepl("^[A-Za-z]", desc_content[i]) && !grepl("^\\s", desc_content[i])) {
-      imports_end <- i - 1
-      break
-    }
-    if (i == length(desc_content)) {
-      imports_end <- length(desc_content)
-    }
-  }
+  imports_packages <- parse_packages(
+    desc_content,
+    imports_bounds$start,
+    imports_bounds$end
+  )
+  imports_names <- sapply(imports_packages, `[[`, "name")
+  has_cli <- "cli" %in% imports_names
 
-  imports_section <- desc_content[imports_start:imports_end]
-  has_cli <- any(grepl("\\bcli\\b", imports_section))
+  modified <- FALSE
 
   if (!has_cli) {
     log_message(
-      "Adding cli dependency to {.file {desc_file}}",
+      "Adding {.pkg cli} dependency to {.file {desc_file}}",
       verbose = verbose
     )
+    imports_packages <- c(
+      imports_packages,
+      list(list(name = "cli", full = "cli"))
+    )
+    modified <- TRUE
+  }
 
-    if (imports_end == imports_start) {
-      desc_content[imports_start] <- paste0(
-        desc_content[imports_start], "\n    cli"
-      )
-    } else {
-      desc_content[imports_end] <- paste0(
-        desc_content[imports_end], "\n    cli"
-      )
+  imports_lines <- format_packages(imports_packages)
+  new_imports <- c("Imports:", imports_lines)
+
+  suggests_bounds <- find_section(desc_content, "Suggests")
+  suggests_packages <- NULL
+  if (!is.null(suggests_bounds)) {
+    suggests_packages <- parse_packages(
+      desc_content,
+      suggests_bounds$start,
+      suggests_bounds$end
+    )
+  }
+
+  if (imports_bounds$end < length(desc_content)) {
+    after_imports <- desc_content[(imports_bounds$end + 1):length(desc_content)]
+  } else {
+    after_imports <- character(0)
+  }
+
+  if (!is.null(suggests_bounds) && suggests_bounds$start > imports_bounds$end) {
+    suggests_rel_start <- suggests_bounds$start - imports_bounds$end
+    suggests_rel_end <- suggests_bounds$end - imports_bounds$end
+
+    if (suggests_rel_start <= length(after_imports)) {
+      if (suggests_rel_end < length(after_imports)) {
+        after_imports <- c(
+          if (suggests_rel_start > 1) after_imports[1:(suggests_rel_start - 1)] else character(0),
+          after_imports[(suggests_rel_end + 1):length(after_imports)]
+        )
+      } else {
+        after_imports <- if (suggests_rel_start > 1) {
+          after_imports[1:(suggests_rel_start - 1)]
+        } else {
+          character(0)
+        }
+      }
     }
+  }
 
-    writeLines(desc_content, desc_file)
+  new_content <- desc_content[1:(imports_bounds$start - 1)]
+  new_content <- c(new_content, new_imports)
+
+  if (!is.null(suggests_packages) && length(suggests_packages) > 0) {
+    suggests_lines <- format_packages(suggests_packages)
+    new_suggests <- c("Suggests:", suggests_lines)
+    new_content <- c(new_content, new_suggests)
+  }
+
+  new_content <- c(new_content, after_imports)
+  desc_content <- new_content
+
+  writeLines(desc_content, desc_file)
+
+  if (modified) {
     log_message(
-      "Successfully added {.pkg cli} to {.file {desc_file}}",
+      "Successfully added {.pkg cli} and sorted dependencies in {.file {desc_file}}",
       message_type = "success",
       verbose = verbose
     )
   } else {
     log_message(
-      "{.pkg cli} already present in {.file {desc_file}}",
+      "{.pkg cli} already present, dependencies sorted in {.file {desc_file}}",
       verbose = verbose
     )
   }
 }
 
-.check_pkgdown <- function(pkgdown_file, pkg_name, verbose = TRUE) {
+check_pkgdown <- function(pkgdown_file, pkg_name, verbose = TRUE) {
   if (!file.exists(pkgdown_file)) {
     log_message(
       "{.file {pkgdown_file}} not found",
@@ -679,3 +720,60 @@ generate_content <- function(
     )
   }
 }
+
+find_section <- function(content, section_name) {
+  section_start <- which(grepl(paste0("^", section_name, ":"), content))
+  if (length(section_start) == 0) {
+    return(NULL)
+  }
+  section_start <- section_start[1]
+
+  section_end <- section_start
+  if (section_start < length(content)) {
+    for (i in (section_start + 1):length(content)) {
+      if (grepl("^[A-Za-z]", content[i]) && !grepl("^\\s", content[i])) {
+        section_end <- i - 1
+        break
+      }
+      if (i == length(content)) {
+        section_end <- length(content)
+      }
+    }
+  }
+
+  list(start = section_start, end = section_end)
+}
+
+parse_packages <- function(content, start, end) {
+  section_lines <- content[start:end]
+  section_text <- paste(section_lines, collapse = "\n")
+  section_text <- sub("^[A-Za-z]+:\\s*", "", section_text)
+
+  packages_raw <- strsplit(section_text, ",")[[1]]
+  packages_raw <- trimws(packages_raw)
+  packages_raw <- packages_raw[packages_raw != ""]
+
+  packages <- lapply(packages_raw, function(pkg) {
+    pkg_name <- sub("\\s*\\(.*\\)\\s*$", "", pkg)
+    pkg_name <- trimws(pkg_name)
+    list(name = pkg_name, full = pkg)
+  })
+
+  packages
+}
+
+format_packages <- function(packages) {
+  packages <- packages[order(tolower(sapply(packages, `[[`, "name")))]
+
+  n <- length(packages)
+  lines <- character(n)
+  for (i in seq_len(n)) {
+    if (i < n) {
+      lines[i] <- paste0("    ", packages[[i]]$full, ",")
+    } else {
+      lines[i] <- paste0("    ", packages[[i]]$full)
+    }
+  }
+
+  lines
+}
